@@ -7,7 +7,7 @@ from discopy.monoidal import Sum
 from discopy.rigid import Functor, PRO
 from discopy.quantum.circuit import Circuit, qubit
 from discopy.quantum.gates import (
-    Bra, Ket, Rz, Rx, Ry, CX, CZ, CRz, CRx, format_number)
+    Bra, Ket, Rz, Rx, Ry, CX, CZ, CRz, CRx, Controlled, format_number)
 from discopy.quantum.gates import Scalar as GatesScalar
 from math import pi
 
@@ -26,9 +26,10 @@ class Diagram(tensor.Diagram):
             left, right, ar_factory=Diagram, swap_factory=Swap)
 
     @staticmethod
-    def permutation(perm, dom=None):
+    def permutation(perm, dom=None, inverse=False):
         dom = PRO(len(perm)) if dom is None else dom
-        return monoidal.Diagram.permutation(perm, dom, ar_factory=Diagram)
+        return monoidal.Diagram.permutation(
+            perm, dom, ar_factory=Diagram, inverse=inverse)
 
     @staticmethod
     def cups(left, right):
@@ -72,7 +73,7 @@ class Diagram(tensor.Diagram):
         ...     >> Id(1) @ SWAP @ Id(1) >> X(2, 1, .5) @ X(2, 1, .5)
         >>> graph = bialgebra.to_pyzx()
         >>> assert len(graph.vertices()) == 8
-        >>> assert (graph.inputs, graph.outputs) == ([0, 1], [6, 7])
+        >>> assert (graph.inputs(), graph.outputs()) == ((0, 1), (6, 7))
         >>> from pyzx import VertexType
         >>> assert graph.type(2) == graph.type(3) == VertexType.Z
         >>> assert graph.phase(2) == 2 * .25 and graph.phase(3) == 2 * .75
@@ -93,7 +94,7 @@ class Diagram(tensor.Diagram):
         for i, _ in enumerate(self.dom):
             node, hadamard = graph.add_vertex(VertexType.BOUNDARY), False
             scan.append((node, hadamard))
-            graph.inputs.append(node)
+            graph.set_inputs(graph.inputs() + (node,))
             graph.set_position(node, i, 0)
         for row, (box, offset) in enumerate(zip(self.boxes, self.offsets)):
             if isinstance(box, Spider):
@@ -123,7 +124,7 @@ class Diagram(tensor.Diagram):
             etype = EdgeType.HADAMARD if hadamard else EdgeType.SIMPLE
             graph.add_edge((source, target), etype)
             graph.set_position(target, i, len(self) + 1)
-            graph.outputs.append(target)
+            graph.set_outputs(graph.outputs() + (target,))
         return graph
 
     @staticmethod
@@ -143,8 +144,9 @@ class Diagram(tensor.Diagram):
         ----
 
         Raises :code:`ValueError` if either:
-        * a boundary node is not in :code:`graph.inputs + graph.outputs`,
-        * :code:`set(graph.inputs).intersection(graph.outputs)` is non-empty.
+        * a boundary node is not in :code:`graph.inputs() + graph.outputs()`,
+        * or :code:`set(graph.inputs()).intersection(graph.outputs())`
+          is non-empty.
         """
         from pyzx import VertexType, EdgeType
 
@@ -159,14 +161,14 @@ class Diagram(tensor.Diagram):
                 swaps = Id(target)\
                     @ Diagram.swap(source - target, 1)\
                     @ Id(len(scan) - source - 1)
-                scan = scan[:target] + [node]\
+                scan = scan[:target] + (scan[source],)\
                     + scan[target:source] + scan[source + 1:]
             elif target > source:
                 swaps = Id(source)\
                     @ Diagram.swap(1, target - source)\
                     @ Id(len(scan) - target - 1)
                 scan = scan[:source] + scan[source + 1:target]\
-                    + [node] + scan[target:]
+                    + (scan[source],) + scan[target:]
             else:
                 swaps = Id(len(scan))
             return scan, swaps
@@ -183,21 +185,21 @@ class Diagram(tensor.Diagram):
 
         missing_boundary = any(
             graph.type(node) == VertexType.BOUNDARY
-            and node not in graph.inputs + graph.outputs
+            and node not in graph.inputs() + graph.outputs()
             for node in graph.vertices())
         if missing_boundary:
             raise ValueError
-        duplicate_boundary = set(graph.inputs).intersection(graph.outputs)
+        duplicate_boundary = set(graph.inputs()).intersection(graph.outputs())
         if duplicate_boundary:
             raise ValueError
-        diagram, scan = Id(len(graph.inputs)), graph.inputs
+        diagram, scan = Id(len(graph.inputs())), graph.inputs()
         for node in [v for v in graph.vertices()
-                     if v not in graph.inputs + graph.outputs]:
+                     if v not in graph.inputs() + graph.outputs()]:
             inputs = [v for v in graph.neighbors(node) if v < node
-                      and v not in graph.outputs or v in graph.inputs]
+                      and v not in graph.outputs() or v in graph.inputs()]
             inputs.sort(key=scan.index)
             outputs = [v for v in graph.neighbors(node) if v > node
-                       and v not in graph.inputs or v in graph.outputs]
+                       and v not in graph.inputs() or v in graph.outputs()]
             scan, diagram, offset = make_wires_adjacent(scan, diagram, inputs)
             hadamards = Id(0).tensor(*[
                 H if graph.edge_type((i, node)) == EdgeType.HADAMARD else Id(1)
@@ -205,9 +207,9 @@ class Diagram(tensor.Diagram):
             box = node2box(node, len(inputs), len(outputs))
             diagram = diagram >> Id(offset) @ (hadamards >> box)\
                 @ Id(len(diagram.cod) - offset - len(inputs))
-            scan = scan[:offset] + len(outputs) * [node]\
+            scan = scan[:offset] + len(outputs) * (node,)\
                 + scan[offset + len(inputs):]
-        for target, output in enumerate(graph.outputs):
+        for target, output in enumerate(graph.outputs()):
             node, = graph.neighbors(output)
             etype = graph.edge_type((node, output))
             hadamard = H if etype == EdgeType.HADAMARD else Id(1)
@@ -294,6 +296,46 @@ class Spider(Box):
         return Scalar(pi * gradient)\
             @ type(self)(len(self.dom), len(self.cod), self.phase + .5)
 
+    @classmethod
+    def make_spiders(cls, n_legs_in, n_legs_out, phase=0):
+        """Construct spider using the generators of the Frobenius algebra.
+
+        Example
+        -------
+        >>> from discopy.drawing import equation
+        >>> orig = Z(4, 3, 0.5)
+        >>> decomp = Z.make_spiders(4, 3, 0.5)
+        >>> equation(orig, decomp, symbol='->',
+        ...     path='docs/_static/imgs/spider-decomp.png')
+
+        .. image:: ../../../_static/imgs/spider-decomp.png
+            :align: center
+
+        """
+        if n_legs_out > n_legs_in:
+            return cls.make_spiders(n_legs_out, n_legs_in, -phase).dagger()
+
+        if n_legs_in == 1 and n_legs_out == 0:
+            return cls(1, 0, phase)
+        if n_legs_in == 1 and n_legs_out == 1:
+            if phase == 0:
+                return Id(1)
+            return cls(1, 1, phase)
+
+        if n_legs_out != 1 or phase != 0:
+            return (cls.make_spiders(n_legs_in, 1, 0)
+                    >> cls.make_spiders(1, 1, phase)
+                    >> cls.make_spiders(1, n_legs_out, 0))
+
+        if n_legs_in % 2 == 1:
+            return (cls.make_spiders(n_legs_in - 1, 1)
+                    @ Id(1) >> cls(2, n_legs_out))
+
+        new_in = n_legs_in // 2
+        return (cls.make_spiders(new_in, 1)
+                @ cls.make_spiders(new_in, 1)
+                >> cls(2, n_legs_out))
+
 
 class Z(Spider):
     """ Z spider. """
@@ -373,10 +415,10 @@ def gate2zx(box):
         return Id(0).tensor(*spiders) @ scalar(pow(2, -len(box.bitstring) / 2))
     if isinstance(box, (Rz, Rx)):
         return (Z if isinstance(box, Rz) else X)(1, 1, box.phase)
-    if isinstance(box, CRz):
+    if isinstance(box, Controlled) and box.name.startswith('CRz'):
         return Z(1, 2) @ Z(1, 2, box.phase)\
             >> Id(1) @ (X(2, 1) >> Z(1, 0, -box.phase)) @ Id(1)
-    if isinstance(box, CRx):
+    if isinstance(box, Controlled) and box.name.startswith('CRx'):
         return X(1, 2) @ X(1, 2, box.phase)\
             >> Id(1) @ (Z(2, 1) >> X(1, 0, -box.phase)) @ Id(1)
     if isinstance(box, quantum.CU1):
@@ -386,16 +428,42 @@ def gate2zx(box):
         if box.is_mixed:
             raise NotImplementedError
         return scalar(box.data)
+    if isinstance(box, Controlled) and box.distance != 1:
+        return circuit2zx(box._decompose())
     standard_gates = {
         quantum.H: H,
         quantum.Z: Z(1, 1, .5),
         quantum.X: X(1, 1, .5),
         quantum.Y: Z(1, 1, .5) >> X(1, 1, .5) @ scalar(1j),
+        quantum.S: Z(1, 1, .25),
+        quantum.T: Z(1, 1, .125),
         CZ: Z(1, 2) @ Id(1) >> Id(1) @ Had() @ Id(1) >> Id(1) @ Z(2, 1),
-        CX: Z(1, 2) @ Id(1) >> Id(1) @ X(2, 1)}
+        CX: Z(1, 2) @ Id(1) >> Id(1) @ X(2, 1) @ scalar(2 ** 0.5)}
     return standard_gates[box]
 
 
 circuit2zx = Functor(
     ob={qubit: PRO(1)}, ar=gate2zx,
     ob_factory=PRO, ar_factory=Diagram)
+
+
+def decomp_ar(box):
+    n, m = len(box.dom), len(box.cod)
+    if isinstance(box, X):
+        phase = box.phase
+        if (n, m) in ((1, 0), (0, 1)):
+            return box
+        box = Id().tensor(*[H] * n) >> Z(n, m, phase) >> Id().tensor(*[H] * m)
+        return decomp(box)
+    if isinstance(box, Z):
+        phase = box.phase
+        if (n, m) == (0, 1):
+            return X(0, 1, phase) >> H
+        if (n, m) == (1, 0):
+            return X(1, 0, phase) << H
+        rot = Id(1) if phase == 0 else Z(1, 1, phase)
+        return Z.make_spiders(n, 1) >> rot >> Z.make_spiders(1, m)
+    return box
+
+
+decomp = Functor(ob=lambda x: x, ar=decomp_ar)
