@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import pytest
 from unittest.mock import Mock
 from functools import partial
 from pytest import raises
@@ -155,7 +156,7 @@ def test_Circuit_to_pennylane(capsys):
 
     x, y, z = sympy.symbols('x y z')
     symbols = [x, y, z]
-    weights = [torch.tensor([1.]), torch.tensor([2.]), torch.tensor([3.])]
+    weights = [torch.tensor(1.), torch.tensor(2.), torch.tensor(3.)]
 
     var_circ = Circuit(
         dom=qubit ** 0, cod=qubit, boxes=[
@@ -167,7 +168,8 @@ def test_Circuit_to_pennylane(capsys):
             3, 2, 0, 0, 0, 0, 0, 0, 1, 0])
 
     p_var_circ = var_circ.to_pennylane()
-    p_var_circ.draw(symbols, weights)
+    p_var_circ.initialise_concrete_params(symbols, weights)
+    p_var_circ.draw()
 
     captured = capsys.readouterr()
     assert captured.out == \
@@ -180,13 +182,14 @@ def test_Circuit_to_pennylane(capsys):
     var_f = var_circ.lambdify(*symbols)
     conc_circ = var_f(*[a.item() for a in weights])
 
-    assert np.allclose(p_var_circ.eval(symbols, weights).numpy(),
+    assert np.allclose(p_var_circ.eval().numpy(),
                        conc_circ.eval().array)
 
     p_var_circ_prob = var_circ.to_pennylane(probabilities=True)
+    p_var_circ_prob.initialise_concrete_params(symbols, weights)
     conc_circ_prob = (conc_circ >> Measure())
 
-    assert (np.allclose(p_var_circ_prob.eval(symbols, weights).numpy(),
+    assert (np.allclose(p_var_circ_prob.eval().numpy(),
                         conc_circ_prob.eval().array))
 
 
@@ -234,19 +237,113 @@ def test_pennylane_parameterized_ops():
         assert np.allclose(disco, plane, atol=10e-5)
 
 
-def test_pennylane_update_post_selection():
+def test_pennylane_devices():
     bell_state = Circuit.caps(qubit, qubit)
     bell_effect = bell_state[::-1]
     snake = (bell_state @ Id(1) >> Bra(0) @ bell_effect)[::-1]
-    p_circ = snake.to_pennylane()
 
-    assert p_circ.post_selection == {0: 0, 1: 0}
-    assert p_circ._valid_states == [0, 1]
+    # Honeywell backend only compatible when `probabilities=True`
+    h_backend = {'backend': 'honeywell.hqs', 'device': 'H1-1E'}
+    h_circ = snake.to_pennylane(probabilities=True, backend_config=h_backend)
+    assert h_circ._device is not None
+    with raises(ValueError):
+        h_circ = snake.to_pennylane(backend_config=h_backend)
 
-    p_circ.post_selection = {0: 0, 2: 0}
+    # Device must be specified when using Honeywell backend
+    h_backend_corrupt = {'backend': 'honeywell.hqs'}
+    with raises(ValueError):
+        h_circ = snake.to_pennylane(probabilities=True,
+                                    backend_config=h_backend_corrupt)
 
-    assert p_circ.post_selection == {0: 0, 2: 0}
-    assert p_circ._valid_states == [0, 2]
+    aer_backend = {'backend': 'qiskit.aer',
+                   'device': 'aer_simulator_statevector'}
+    aer_circ = snake.to_pennylane(backend_config=aer_backend)
+    assert aer_circ._device is not None
+
+    # `aer_simulator` is not compatible with state outputs
+    aer_backend_corrupt = {'backend': 'qiskit.aer', 'device': 'aer_simulator'}
+    with raises(ValueError):
+        aer_circ = snake.to_pennylane(backend_config=aer_backend_corrupt)
+
+
+def test_pennylane_uninitialized():
+    x, y, z = sympy.symbols('x y z')
+    var_circ = Circuit(
+        dom=qubit ** 0, cod=qubit, boxes=[
+            Ket(0), Rx(0.552), Rz(x), Rx(0.917), Ket(0, 0, 0), H, H, H,
+            CRz(0.18), CRz(y), CX, H, sqrt(2), Bra(0, 0), Ket(0),
+            Rx(0.446), Rz(0.256), Rx(z), CX, H, sqrt(2), Bra(0, 0)],
+        offsets=[
+            0, 0, 0, 0, 0, 0, 1, 2, 0, 1, 2, 2,
+            3, 2, 0, 0, 0, 0, 0, 0, 1, 0])
+    p_var_circ = var_circ.to_pennylane()
+
+    with raises(ValueError):
+        p_var_circ.draw()
+
+    with raises(ValueError):
+        p_var_circ.eval()
+
+
+def test_pennylane_parameter_reference():
+    x = sympy.symbols('x')
+    p = torch.nn.Parameter(torch.tensor(1.))
+
+    circ = Rx(x)
+    p_circ = circ.to_pennylane()
+    p_circ.initialise_concrete_params([x], [p])
+
+    with torch.no_grad():
+        p.add_(1.)
+
+    assert p_circ._concrete_params[0][0] == p
+
+    with torch.no_grad():
+        p.add_(-2.)
+
+    assert p_circ._concrete_params[0][0] == p
+
+
+def test_pennylane_gradient_methods():
+    x, y, z = sympy.symbols('x y z')
+    symbols = [x, y, z]
+
+    var_circ = Circuit(
+        dom=qubit ** 0, cod=qubit, boxes=[
+            Ket(0), Rx(0.552), Rz(x), Rx(0.917), Ket(0, 0, 0), H, H, H,
+            CRz(0.18), CRz(y), CX, H, sqrt(2), Bra(0, 0), Ket(0),
+            Rx(0.446), Rz(0.256), Rx(z), CX, H, sqrt(2), Bra(0, 0)],
+        offsets=[
+            0, 0, 0, 0, 0, 0, 1, 2, 0, 1, 2, 2,
+            3, 2, 0, 0, 0, 0, 0, 0, 1, 0])
+
+    for diff_method in ['backprop', 'parameter-shift', 'finite-diff']:
+
+        weights = [torch.tensor(1., requires_grad=True),
+                   torch.tensor(2., requires_grad=True),
+                   torch.tensor(3., requires_grad=True)]
+
+        p_var_circ = var_circ.to_pennylane(probabilities=True,
+                                           diff_method=diff_method)
+        p_var_circ.initialise_concrete_params(symbols, weights)
+
+        loss = p_var_circ.eval().norm(dim=0, p=2)
+        loss.backward()
+        assert weights[0].grad is not None
+
+    for diff_method in ['backprop']:
+
+        weights = [torch.tensor(1., requires_grad=True),
+                   torch.tensor(2., requires_grad=True),
+                   torch.tensor(3., requires_grad=True)]
+
+        p_var_circ = var_circ.to_pennylane(probabilities=False,
+                                           diff_method=diff_method)
+        p_var_circ.initialise_concrete_params(symbols, weights)
+
+        loss = p_var_circ.eval().norm(dim=0, p=2)
+        loss.backward()
+        assert weights[0].grad is not None
 
 
 def test_Sum_from_tk():
@@ -782,3 +879,49 @@ def test_circuit_chaining():
         Id(1).CRx(0.7, 1, 0)
     with raises(ValueError):
         Id(2).X(999)
+
+
+@pytest.mark.parametrize('x,y', [(0, 1), (0, 2), (1, 0), (2, 0), (5, 0)])
+def test_CX_decompose(x, y):
+    n = abs(x - y) + 1
+    binary_mat = np.eye(n, dtype=int)
+    binary_mat[y] = np.bitwise_xor(binary_mat[x], binary_mat[y])
+
+    N = 1 << n
+    unitary_mat = np.zeros(shape=(N, N))
+    for i in range(N):
+        bits = index2bitstring(i, n)
+        v = bitstring2index(binary_mat @ bits % 2)
+        unitary_mat[i][v] = 1
+
+    # take transpose because tensor axes follow diagrammatic order
+    out = Id(n).CX(x, y).eval().array.reshape(N, N).T
+    # but CX matrices are self transpose
+    assert (out == out.T).all()
+    assert (out == unitary_mat).all()
+
+
+@pytest.mark.parametrize('x,y, z', [(0, 1, 2), (0, 2, 4),
+                                    (0, 4, 2), (4, 2, 0),
+                                    (0, 4, 1), (4, 0, 1)])
+def test_CCX_decompose(x, y, z):
+
+    n = max(x, y, z) - min(x, y, z) + 1
+    N = 1 << n
+
+    unitary_mat = np.zeros(shape=(N, N))
+
+    for i in range(N):
+        bits = list(index2bitstring(i, n))
+        bits[z] = (bits[x] & bits[y]) ^ bits[z]
+        v = bitstring2index(bits)
+        unitary_mat[i][v] = 1
+
+    # take transpose because tensor axes follow diagrammatic order
+    out = Id(n).CCX(x, y, z).eval().array.reshape(N, N).T
+
+    np.set_printoptions(threshold=3000)
+
+    print(unitary_mat.real)
+
+    assert (out == unitary_mat).all()
