@@ -1,9 +1,33 @@
 from pytest import raises
-from unittest.mock import Mock
 import numpy as np
+import tensornetwork as tn
 from discopy.tensor import *
 
 
+def test_backend():
+    import jax.numpy
+    import torch
+    import tensorflow.experimental.numpy as tnp
+    assert Tensor.np.module == np
+    with Tensor.backend('jax'):
+        assert Tensor.np.module == jax.numpy
+        with Tensor.backend('pytorch'):
+            assert Tensor.np.module == torch
+            with Tensor.backend('tensorflow'):
+                assert Tensor.np.module == tnp
+        assert Tensor.np.module == jax.numpy
+    assert Tensor.np.module == np
+
+    with raises(ValueError):
+        Tensor.set_backend('nonexistent')
+
+
+def test_Tensor_repr_with_tf():
+    with Tensor.backend('tensorflow'):
+        alice = Tensor(Dim(1), Dim(2), [1, 2])
+        assert repr(alice) == 'Tensor(dom=Dim(1), cod=Dim(2), array=[1, 2])'
+
+
 def test_Dim():
     with raises(TypeError):
         Dim('a')
@@ -42,6 +66,32 @@ def test_Tensor():
     assert Tensor.id(Dim(2)).then(*(m, m)) == m >> m.dagger()
 
 
+def test_Spider_to_tn():
+    d = Dim(2)
+    tensor = Spider(1, 1, d) >> Spider(1, 2, d) >> Spider(2, 0, d)
+    result = tensor.eval(contractor=tn.contractors.auto).array
+    assert all(result == np.array([1, 1]))
+
+
+def test_Spider_to_tn_pytorch():
+    try:
+        import torch
+        Tensor.np = torch
+        torch.array = torch.as_tensor
+        tn.set_default_backend('pytorch')
+
+        d = Dim(2)
+
+        alice = Box("Alice", Dim(1), d,
+                    torch.as_tensor([1., 2.]).requires_grad_(True))
+        tensor = alice >> Spider(1, 2, d) >> Spider(2, 0, d)
+        result = tensor.eval(contractor=tn.contractors.auto).array
+        assert result.item() == 3
+    finally:
+        Tensor.np = np
+        tn.set_default_backend('numpy')
+
+
 def test_Tensor_cups():
     assert np.all(Tensor.cups(Dim(2), Dim(2)).array == np.identity(2))
     with raises(TypeError):
@@ -139,7 +189,9 @@ def test_Tensor_iter():
     v = Tensor(Dim(1), Dim(2), [0, 1])
     assert list(v) == [0, 1]
     s = Tensor(Dim(1), Dim(1), [1])
-    assert list(s) == [1]
+    with raises(TypeError):
+        # how does one iterate over a scalar?
+        list(s)
 
 
 def test_Tensor_subs():
@@ -159,7 +211,7 @@ def test_Diagram_cups_and_caps():
 def test_Diagram_swap():
     x, y, z = Dim(2), Dim(3), Dim(4)
     assert Diagram.swap(x, y @ z) == \
-           (Swap(x, y) @ Id(z)) >> (Id(y) @ Swap(x, z))
+        (Swap(x, y) @ Id(z)) >> (Id(y) @ Swap(x, z))
 
 
 def test_Box():
@@ -225,7 +277,13 @@ def test_Tensor_adjoint_eval():
 
 
 def test_non_numpy_eval():
-    Tensor.np = Mock(__package__='pytorch')
+    import torch
+    Tensor.np = torch
     with raises(Exception):
         Swap(Dim(2), Dim(2)).eval()
     Tensor.np = np
+
+
+def test_Tensor_array():
+    box = Box("box", Dim(2), Dim(2), None)
+    assert box.array is None
